from typing import List
from vantage6.client.filter import post_filtering
from vantage6.common.client.client_base import ClientBase


class StoreUserSubClient(ClientBase.SubClient):
    """Subclient for the user registrations in the algorithm store."""

    @post_filtering(iterable=True)
    def list(
        self,
        username: str = None,
        role: int = None,
        page: int = 1,
        per_page: int = 10,
    ) -> List[dict]:
        """
        List algorithms

        Parameters
        ----------
        username : str, optional
            Filter by username (with LIKE operator).
        role : int, optional
            Filter by role id.
        field : str, optional
            Which data field to keep in the result. For instance, "field='username'"
            will only return the username of the registered users. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['username', 'id']" will only return the username and id of the
            registered users. Default is None.
        filter_ : tuple, optional
            Filter the result on key-value pairs. For instance,
            "filter_=('username', 'my_username')" will only return the registered
            users with the username 'my_username'. Default is None.
        filters : list[tuple], optional
            Filter the result on multiple key-value pairs. For instance,
            "filters=[('username', 'my_username'), ('id', 1)]" will only return the
            registered users with the username 'my_username' and id 1. Default is None.
        page : int, optional
            Page number for pagination (default=1)
        per_page : int, optional
            Number of items per page (default=10)

        Returns
        -------
        list[dict]
            List of user registrations
        """
        params = {
            "username": username,
            "role_id": role,
            "page": page,
            "per_page": per_page,
        }
        return self.parent.request(
            "user",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            params=params,
        )

    @post_filtering(iterable=False)
    def get(self, id_: int) -> dict:
        """
        Get a user registration by id

        Parameters
        ----------
        id_ : int
            The id of the user registration
        field : str, optional
            Which data field to keep in the result. For instance, "field='username'"
            will only return the username of the user. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['username', 'id']" will only return the username and id of the
            user. Default is None.

        Returns
        -------
        dict
            The user
        """
        return self.parent.request(
            f"user/{id_}",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )

    @post_filtering(iterable=False)
    def register(self, username: str, roles: List[int]) -> dict:
        """
        Register a vantage6 user in this algorithm store.

        Parameters
        ----------
        username : str
            The username of the user
        roles : list[int]
            The roles of the user in this algorithm store
        field : str, optional
            Which data field to keep in the returned dict. For instance,
            "field='username'" will only return the username of the user. Default is
            None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['username', 'id']" will only return the username and id of the
            user. Default is None.

        Returns
        -------
        dict
            The user registration
        """
        data = {
            "username": username,
            "roles": roles,
        }
        return self.parent.request(
            "user",
            method="POST",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=data,
        )

    @post_filtering(iterable=False)
    def update(self, id_: int, roles: List[int]) -> dict:
        """
        Update a user registration by id

        Parameters
        ----------
        id_ : int
            The id of the user registration
        roles : list[int]
            The new roles of the user in this algorithm store
        field : str, optional
            Which data field to keep in the returned dict. For instance,
            "field='username'" will only return the username of the user. Default is
            None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['username', 'id']" will only return the username and id of the
            user. Default is None.

        Returns
        -------
        dict
            The updated user registration
        """
        data = {
            "roles": roles,
        }
        return self.parent.request(
            f"user/{id_}",
            method="PATCH",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=data,
        )

    def delete(self, id_: int) -> None:
        """
        Delete a user registration by id

        Parameters
        ----------
        id_ : int
            The id of the user registration
        """
        res = self.parent.request(
            f"user/{id_}",
            method="DELETE",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )
        self.parent.log.info(f"--> {res.get('msg')}")



from typing import List
from vantage6.client.filter import post_filtering
from vantage6.common.client.client_base import ClientBase


class ReviewSubClient(ClientBase.SubClient):
    """Subclient for the reviews from the algorithm store."""

    @post_filtering(iterable=True)
    def list(
        self,
        algorithm: int = None,
        reviewer: int = None,
        under_review: bool = None,
        reviewed: bool = None,
        approved: bool = None,
        rejected: bool = None,
        page: int = 1,
        per_page: int = 10,
    ) -> List[dict]:
        """
        List reviews

        Parameters
        ----------
        algorithm : int, optional
            Filter by algorithm id.
        reviewer : int, optional
            Filter by user id of the reviewer.
        under_review : bool, optional
            Filter by under review status.
        reviewed : bool, optional
            Filter by reviewed (either approve or rejected).
        approved : bool, optional
            Filter by approved status.
        rejected : bool, optional
            Filter by rejected status.
        field : str, optinal
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the reviews. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            reviews. Default is None.
        filter_ : tuple, optional
            Filter the result on key-value pairs. For instance,
            "filter_=('name', 'my_name')" will only return the reviews with the
            name 'my_name'. Default is None.
        filters : list[tuple], optional
            Filter the result on multiple key-value pairs. For instance,
            "filters=[('name', 'my_name'), ('id', 1)]" will only return the
            reviews with the name 'my_name' and id 1. Default is None.
        page : int, optional
            Page number for pagination (default=1)
        per_page : int, optional
            Number of items per page (default=10)

        Returns
        -------
        list[dict]
            List of reviews
        """
        params = {
            "algorithm_id": algorithm,
            "reviewer_id": reviewer,
            "page": page,
            "per_page": per_page,
        }
        if under_review is not None:
            params["under_review"] = under_review
        if reviewed is not None:
            params["reviewed"] = reviewed
        if approved is not None:
            params["approved"] = approved
        if rejected is not None:
            params["rejected"] = rejected
        return self.parent.request(
            "review",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            params=params,
        )

    @post_filtering(iterable=False)
    def get(self, id_: int) -> dict:
        """
        Get a review by its id.

        Parameters
        ----------
        id_ : int
            The id of the review.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the review. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            review. Default is None.

        Returns
        -------
        dict
            The review.
        """
        return self.parent.request(
            f"review/{id_}",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )

    @post_filtering(iterable=False)
    def create(self, algorithm: int, reviewer: int) -> dict:
        """
        Assign an algorithm to be reviewed by a particular user.

        Parameters
        ----------
        algorithm : int
            The id of the algorithm.
        reviewer : int
            The user id for the reviewer.
        field : str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the review. Default is None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the review.
            Default is None.

        Returns
        -------
        dict
            The created review.
        """
        return self.parent.request(
            "review",
            method="post",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json={"algorithm_id": algorithm, "reviewer_id": reviewer},
        )

    def delete(self, id_: int) -> None:
        """
        Delete a review.

        Parameters
        ----------
        id_ : int
            The id of the review.
        """
        res = self.parent.request(
            f"review/{id_}",
            method="delete",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )
        self.parent.log.info(f"--> {res.get('msg')}")

    def approve(self, id_: int, comment: str = None) -> dict:
        """
        Approve a review.

        Parameters
        ----------
        id_ : int
            The id of the review.
        comment : str
            A comment for the approval of the review. Optional.

        Returns
        -------
        dict
            The approved review.
        """
        body = {}
        if comment:
            body["comment"] = comment
        return self.parent.request(
            f"review/{id_}/approve",
            method="post",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=body,
        )

    def reject(self, id_: int, comment: str = None) -> dict:
        """
        Reject a review.

        Parameters
        ----------
        id_ : int
            The id of the review.
        comment : str
            A comment for the rejection of the review. Optional.

        Returns
        -------
        dict
            The rejected review.
        """
        body = {}
        if comment:
            body["comment"] = comment
        return self.parent.request(
            f"review/{id_}/reject",
            method="post",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=body,
        )



from typing import List
from vantage6.client.filter import post_filtering
from vantage6.common.client.client_base import ClientBase


class AlgorithmSubClient(ClientBase.SubClient):
    """Subclient for the algorithms from the algorithm store."""

    @post_filtering(iterable=False)
    def get(self, id_: int) -> dict:
        """Get an algorithm by its id.

        Parameters
        ----------
        id_ : int
            The id of the algorithm.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the algorithm. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            algorithm. Default is None.

        Returns
        -------
        dict
            The algorithm.
        """
        return self.parent.request(
            f"algorithm/{id_}",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )

    @post_filtering(iterable=True)
    def list(
        self,
        name: str = None,
        description: str = None,
        image: str = None,
        partitioning: str = None,
        v6_version: str = None,
        awaiting_reviewer_assignment: bool = None,
        under_review: bool = None,
        in_review_process: bool = None,
        invalidated: bool = None,
        page: int = 1,
        per_page: int = 10,
    ) -> list[dict]:
        """
        List algorithms.

        By default, only approved algorithms are returned.

        Parameters
        ----------
        name : str
            Filter by name (with LIKE operator).
        description : str
            Filter by description (with LIKE operator).
        image : str
            Filter by image (with LIKE operator).
        partitioning : str
            Filter by partitioning (horizontal or vertical).
        v6_version : str
            Filter by version (with LIKE operator).
        awaiting_reviewer_assignment : bool
            Filter by whether the algorithm is awaiting reviewer assignment.
        under_review: bool
            Filter by whether the algorithm is under review.
        in_review_process : bool
            Filter by whether the algorithm is in the review process, i.e. either
            awaiting reviewer assignment or under review.
        invalidated : bool
            Filter by whether the algorithm is invalidated.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the algorithms. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            algorithms. Default is None.
        filter_: tuple, optional
            Filter the result on key-value pairs. For instance,
            "filter_=('name', 'algorithm1')" will only return the algorithms with
            the name 'algorithm1'. Default is None.
        filters: list[tuple], optional
            Filter the result on multiple key-value pairs. For instance,
            "filters=[('name', 'algorithm1'), ('id', 1)]" will only return the
            algorithms with the name 'algorithm1' and id 1. Default is None.
        page : int, optional
            The page number to retrieve.
        per_page : int, optional
            The number of items to retrieve per page.

        Returns
        -------
        list[dict]
            List of algorithms
        """
        params = {
            "name": name,
            "description": description,
            "image": image,
            "partitioning": partitioning,
            "v6_version": v6_version,
            "page": page,
            "per_page": per_page,
        }
        if awaiting_reviewer_assignment is not None:
            params["awaiting_reviewer_assignment"] = awaiting_reviewer_assignment
        if under_review is not None:
            params["under_review"] = under_review
        if in_review_process is not None:
            params["in_review_process"] = in_review_process
        if invalidated is not None:
            params["invalidated"] = invalidated
        return self.parent.request(
            "algorithm",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            params=params,
        )

    @post_filtering(iterable=False)
    def create(
        self,
        name: str,
        description: str,
        image: str,
        partitioning: str,
        vantage6_version: str,
        code_url: str,
        functions: List[dict],
        documentation_url: str = None,
    ) -> dict:
        """
        Add an algorithm to the algorithm store

        Parameters
        ----------
        name : str
            Name of the algorithm
        description : str
            Description of the algorithm
        image : str
            Docker image of the algorithm
        partitioning : str
            Partitioning of the algorithm (horizontal or vertical)
        vantage6_version : str
            Vantage6 version of the algorithm
        code_url : str
            URL to the repository containing the algorithm code
        functions : list[dict]
            List of functions of the algorithm. Each function is a dict with
            the following keys:
            - name: str
                Name of the function
            - display_name: str, optional
                Display name of the function
            - description: str, optional
                Description of the function
            - type: string
                Type of the function (central or federated)
            - standalone: bool
                Whether this function produces useful results when running it by itself
            - databases: list[dict]
                List of databases of the function. Each database is a dict with
                the following keys:
                - name: str
                    Name of the database
                - description: str, optional
                    Description of the database
            - arguments: list[dict]
                List of arguments of the function. Each argument is a dict with
                the following keys:
                - name: str
                    Name of the argument
                - display_name: str, optional
                    Display name of the argument
                - description: str, optional
                    Description of the argument
                - type: str
                    Type of the argument. Can be 'string', 'integer', 'float',
                    'boolean', 'json', 'column', 'organization' or 'organizations'
                - allowed_values: list[str | int | float], optional
                    An optional list of allowed values for the argument. The type of
                    the values should match the 'type' field, e.g. if 'type' is
                    'integer', the allowed values should be a list of integers.
                - has_default_value: bool, optional
                    Whether the argument has a default value. Default is False.
                - default_value: str | int | float | boolean | list, optional
                    Default value of the argument. Only required if 'has_default_value'
                    is true. The type should match the 'type' field, e.g. if 'type' is
                    'integer', 'default_value' should be an integer.
                - conditional_on: str, optional
                    Name of the argument that this argument is conditional on.
                - conditional_comparator: str, optional
                    Comparator to use for the conditional argument. Can be '==', '!=',
                    '>', '<', '>=', or '<='.
                - conditional_value: str | int | float | boolean, optional
                    Value to compare the conditional argument to.
                - is_frontend_only: bool, optional
                    Whether the argument should should only be shown in the UI form, or
                    also be passed to the algorithm. Default is False.
            - ui_visualizations: list[dict]
                List of UI visualizations of the function. Each visualization
                is a dict with the following keys:
                - name: str
                    Name of the visualization
                - description: str, optional
                    Description of the visualization
                - type: str
                    Type of the visualization. Can be 'table'
                - schema: dict, optional
                    Visualization details. For example, for a table, the schema
                    could be a dict with the following keys:
                    - location: list[str]
                        Where data to be visualized are stored in the results. For
                        example, if the data is stored in the 'data' key of the
                        results, the location would be ['data'].
                    - columns: list[str]
                        List of column names to visualize
        documentation_url : str, optional
            URL to the documentation of the algorithm
        field : str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the algorithm. Default is None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the algorithm.

        Returns
        -------
        dict
            The created algorithm
        """
        body = {
            "name": name,
            "image": image,
            "partitioning": partitioning,
            "vantage6_version": vantage6_version,
            "functions": functions,
            "description": description,
            "code_url": code_url,
        }
        if documentation_url:
            body["documentation_url"] = documentation_url
        return self.parent.request(
            "algorithm",
            method="post",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=body,
        )

    def delete(self, id_: int) -> None:
        """
        Delete an algorithm from the algorithm store

        Parameters
        ----------
        id_ : int
            Id of the algorithm
        """
        res = self.parent.request(
            f"algorithm/{id_}",
            method="delete",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )
        self.parent.log.info(f"--> {res.get('msg')}")

    @post_filtering(iterable=False)
    def invalidate(self, id_: int) -> dict:
        """
        Invalidate an algorithm in the algorithm store. An invalidated algorithm
        is marked as outdated but is not deleted - you can therefore still view it
        and link that to the results of executed analyses.

        Parameters
        ----------
        id_ : int
            Id of the algorithm
        field : str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the algorithm. Default is None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the algorithm.

        Returns
        -------
        dict
            The invalidated algorithm
        """
        return self.parent.request(
            f"algorithm/{id_}/invalidate",
            method="post",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )

    @post_filtering(iterable=False)
    def update(
        self,
        id_: int,
        name: str = None,
        description: str = None,
        image: str = None,
        partitioning: str = None,
        vantage6_version: str = None,
        code_url: str = None,
        documentation_url: str = None,
        functions: List[dict] = None,
        refresh_digest: bool = None,
    ) -> dict:
        """
        Update an algorithm in the algorithm store

        Parameters
        ----------
        id_ : int
            Id of the algorithm
        name : str
            Name of the algorithm
        description : str
            Description of the algorithm
        image : str
            Docker image of the algorithm
        partitioning : str
            Partitioning of the algorithm (horizontal or vertical)
        vantage6_version : str
            Vantage6 version of the algorithm
        code_url : str
            URL to the repository containing the algorithm code
        documentation_url : str, optional
            URL to the documentation of the algorithm
        functions : list[dict], optional
            List of functions of the algorithm. If given, all old functions are replaced
            with these new ones. Each function is a dict with the following keys:
            - name: str
                Name of the function
            - display_name: str, optional
                Display name of the function
            - description: str, optional
                Description of the function
            - type: string
                Type of the function (central or federated)
            - standalone: bool
                Whether this function produces useful results when running it by itself
            - databases: list[dict]
                List of databases of the function. Each database is a dict with
                the following keys:
                - name: str
                    Name of the database
                - description: str, optional
                    Description of the database
            - arguments: list[dict]
                List of arguments of the function. Each argument is a dict with
                the following keys:
                - name: str
                    Name of the argument
                - display_name: str, optional
                    Display name of the argument
                - description: str, optional
                    Description of the argument
                - type: str
                    Type of the argument. Can be 'string', 'integer', 'float',
                    'boolean', 'json', 'column', 'organization' or 'organizations'
                - allowed_values: list[str], optional
                    An optional list of allowed values for the argument
                - has_default_value: bool, optional
                    Whether the argument has a default value. Default is False.
                - default_value: str | int | float | boolean | list, optional
                    Default value of the argument. Only required if 'has_default_value'
                    is true. The type should match the 'type' field, e.g. if 'type' is
                    'integer', 'default_value' should be an integer.
                - conditional_on: str, optional
                    Name of the argument that this argument is conditional on.
                - conditional_comparator: str, optional
                    Comparator to use for the conditional argument. Can be '==', '!=',
                    '>', '<', '>=', or '<='.
                - conditional_value: str | int | float | boolean, optional
                    Value to compare the conditional argument to.
                - is_frontend_only: bool, optional
                    Whether the argument should should only be shown in the UI form, or
                    also be passed to the algorithm. Default is False.
            - ui_visualizations: list[dict]
                List of UI visualizations of the function. Each visualization
                is a dict with the following keys:
                - name: str
                    Name of the visualization
                - description: str, optional
                    Description of the visualization
                - type: str
                    Type of the visualization. Can be 'table'
                - schema: dict, optional
                    Visualization details. For example, for a table, the schema
                    could be a dict with the following keys:
                    - location: list[str]
                        Where data to be visualized are stored in the results. For
                        example, if the data is stored in the 'data' key of the
                        results, the location would be ['data'].
                    - columns: list[str]
                        List of column names to visualize
            refresh_digest : bool, optional
                Whether to refresh the digest of the algorithm. This is useful
                when the algorithm image has been updated before the algorithm was
                in review.
            field : str, optional
                Which data field to keep in the returned dict. For instance,
                "field='name'" will only return the name of the algorithm. Default is
                None.
            fields : list[str], optional
                Which data fields to keep in the returned dict. For instance,
                "fields=['name', 'id']" will only return the name and id of the
                algorithm. Default is None.

        Returns
        -------
        dict
            The updated algorithm
        """
        body = {
            "name": name,
            "description": description,
            "image": image,
            "partitioning": partitioning,
            "vantage6_version": vantage6_version,
            "code_url": code_url,
            "documentation_url": documentation_url,
            "functions": functions,
            "refresh_digest": refresh_digest,
        }
        body = self._clean_update_data(body)
        return self.parent.request(
            f"algorithm/{id_}",
            method="PATCH",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=body,
        )



from vantage6.client.filter import post_filtering
from vantage6.client.subclients.store.review import ReviewSubClient
from vantage6.client.subclients.store.role import StoreRoleSubClient
from vantage6.client.subclients.store.rule import StoreRuleSubClient
from vantage6.client.subclients.store.user import StoreUserSubClient
from vantage6.client.subclients.store.policy import PolicySubClient
from vantage6.common.client.client_base import ClientBase


class AlgorithmStoreSubClient(ClientBase.SubClient):
    """Subclient for the algorithm store."""

    def __init__(self, parent: ClientBase):
        super().__init__(parent)
        self.url = None
        self.store_id = None

        self.base_client = self.parent

        self.role = StoreRoleSubClient(self)
        self.rule = StoreRuleSubClient(self)
        self.user = StoreUserSubClient(self)
        self.policy = PolicySubClient(self)
        self.review = ReviewSubClient(self)

    def set(self, id_: int) -> dict:
        """ "
        Set the algorithm store to use for the client.

        Parameters
        ----------
        id_ : int
            The id of the algorithm store.

        Returns
        -------
        dict
            The algorithm store.
        """
        store = self.get(id_)
        try:
            self.url = f"{store['url']}/api"
            self.store_id = id_
        except KeyError:
            self.parent.log.error("Algorithm store URL could not be set.")
        return store

    @post_filtering(iterable=False)
    def get(self, id_: int) -> dict:
        """Get an algorithm store by its id.

        Parameters
        ----------
        id_ : int
            The id of the algorithm store.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the algorithm store. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            algorithm store. Default is None.

        Returns
        -------
        dict
            The algorithm store.
        """
        return self.parent.request(f"algorithmstore/{id_}")

    @post_filtering(iterable=True)
    def list(
        self,
        name: str = None,
        url: str = None,
        collaboration: int = None,
        page: int = 1,
        per_page: int = 10,
    ) -> list[dict]:
        """List all algorithm stores.

        Parameters
        ----------
        name : str, optional
            Filter by name (with LIKE operator)
        url : str, optional
            Filter by algorithm store url (with LIKE operator)
        collaboration : int, optional
            Filter by collaboration id. If not given and client.setup_collaboration()
            was called, the collaboration id from the setup is used. Otherwise, all
            algorithm stores are returned.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the algorithm stores. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            algorithm stores. Default is None.
        page : int, optional
            The page number to retrieve.
        per_page : int, optional
            The number of items to retrieve per page.

        Returns
        -------
        list[dict]
            The algorithm stores.
        """
        if collaboration is None:
            collaboration = self.parent.collaboration_id
        params = {
            "name": name,
            "url": url,
            "collaboration_id": collaboration,
            "page": page,
            "per_page": per_page,
        }
        return self.parent.request("algorithmstore", params=params)

    @post_filtering(iterable=False)
    def create(
        self,
        algorithm_store_url: str,
        name: str,
        collaboration: int | None = None,
        all_collaborations: bool = False,
        force: bool = False,
    ) -> dict:
        """
        Link an algorithm store to one or more collaborations.

        Parameters
        ----------
        algorithm_store_url : str
            The url of the algorithm store, including the API path.
        name : str
            The name of the algorithm store.
        collaboration : int, optional
            The id of the collaboration to link the algorithm store to. If not given
            and client.setup_collaboration() was called, the collaboration id from the
            setup is used. If neither is the case, all_collaborations must be set to
            True explicitly.
        all_collaborations : bool, optional
            If True, the algorithm store is linked to all collaborations. If False,
            the collaboration_id must be given.
        force : bool, optional
            If True, the algorithm store will be linked to the collaboration even for
            localhost urls - which is not recommended in production scenarios for
            security reasons.
        field : str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the algorithm store. Default is None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the algorithm
            store. Default is None.

        Returns
        -------
        dict
            The algorithm store.
        """
        if all_collaborations:
            collaboration = None
        elif collaboration is None and self.parent.collaboration_id is not None:
            collaboration = self.parent.collaboration_id
        elif collaboration is None:
            self.parent.log.error(
                "No collaboration given and default collaboration is not set. "
                "Please provide a collaboration, set a default collaboration, or make "
                "the algorithm store available to all collaborations with "
                "all_collaborations=True."
            )
            return
        data = {
            "algorithm_store_url": algorithm_store_url,
            "name": name,
            "force": force,
            "server_url": self.parent.base_path,
        }
        if collaboration is not None:
            data["collaboration_id"] = (collaboration,)
        return self.parent.request("algorithmstore", method="post", json=data)

    @post_filtering(iterable=False)
    def update(
        self,
        id_: int = None,
        name: str = None,
        collaboration: int = None,
        all_collaborations: bool = None,
    ) -> dict:
        """Update an algorithm store.

        Parameters
        ----------
        id_ : int
            The id of the algorithm store. If not given, the algorithm store must be
            set with client.store.set().
        name : str, optional
            The name of the algorithm store.
        collaboration : int, optional
            The id of the collaboration to link the algorithm store to.
        all_collaborations : bool, optional
            If True, the algorithm store is linked to all collaborations. If False,
            the collaboration_id must be given.
        field : str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the algorithm store. Default is None.
        fields : list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the algorithm
            store. Default is None.

        Returns
        -------
        dict
            The updated algorithm store.
        """
        id_ = self.__get_store_id(id_)
        if id_ is None:
            return
        data = {}
        if name is not None:
            data["name"] = name
        if collaboration is not None or all_collaborations:
            data["collaboration_id"] = collaboration
        return self.parent.request(f"algorithmstore/{id_}", method="patch", json=data)

    def delete(self, id_: int = None) -> None:
        """Delete an algorithm store.

        Parameters
        ----------
        id_ : int
            The id of the algorithm store. If not given, the algorithm store must be
            set with client.store.set().
        """
        id_ = self.__get_store_id(id_)
        if id_ is None:
            return
        res = self.parent.request(
            f"algorithmstore/{id_}",
            method="delete",
            params={
                "server_url": self.parent.base_path,
            },
        )
        self.parent.log.info(f"--> {res.get('msg')}")

    def __get_store_id(self, id_: int = None) -> int:
        """
        Get the algorithm store id.

        Parameters
        ----------
        id_ : int
            The id of the algorithm store. If not given, the algorithm store must be
            set with client.store.set().

        Returns
        -------
        int
            The algorithm store id.
        """
        if id_ is None:
            id_ = self.store_id
            if id_ is None:
                self.parent.log.error("No algorithm store id given or set.")
                return
        return id_



from typing import List
from vantage6.client.filter import post_filtering
from vantage6.common.client.client_base import ClientBase


class StudySubClient(ClientBase.SubClient):
    """Subclient for the algorithm store."""

    @post_filtering(iterable=False)
    def get(self, id_: int) -> dict:
        """Get a study by its id.

        Parameters
        ----------
        id_ : int
            The id of the study
        field: str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the study. Default is None.
        fields: list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            study. Default is None.


        Returns
        -------
        dict
            The study
        """
        return self.parent.request(f"study/{id_}")

    @post_filtering()
    def list(
        self,
        name: str | None = None,
        organization: int | None = None,
        collaboration: int | None = None,
        include_organizations: bool = False,
        page: int = 1,
        per_page: int = 20,
    ) -> List[dict]:
        """View your studies

        Parameters
        ----------
        name: str, optional (with LIKE operator)
            Filter studies by name
        organization: int, optional
            Filter studies by organization id
        collaboration: int, optional
            Filter studies by collaboration id
        include_organizations: bool, optional
            Include organizations in the response, by default False
        field: str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the studies. Default is None.
        fields: list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            studies. Default is None.
        filter_: tuple, optional
            Filter the result on key-value pairs. For instance,
            "filter_=('name', 'study1')" will only return the studies with the name
            'study1'. Default is None.
        filters: list[tuple], optional
            Filter the result on multiple key-value pairs. For instance,
            "filters=[('name', 'study1'), ('id', 1)]" will only return the studies
            with the name 'study1' and id 1. Default is None.
        page: int, optional
            Pagination page, by default 1
        per_page: int, optional
            Number of items on a single page, by default 20

        Returns
        -------
        list[dict]
            Containing collabotation information
        """
        params = {
            "page": page,
            "per_page": per_page,
            "name": name,
        }
        if organization is not None:
            params["organization_id"] = organization
        if collaboration is not None:
            params["collaboration_id"] = collaboration
        if include_organizations:
            params["include"] = "organizations"
        return self.parent.request("study", params=params)

    @post_filtering(iterable=False)
    def create(
        self, name: str, organizations: List[int], collaboration: int = None
    ) -> dict:
        """Create new study

        Parameters
        ----------
        name : str
            Name of the study
        organizations : list[int]
            List of organization ids which participate in the study
        collaboration : int | None
            Id of the collaboration the study is part of. If None, the value of
            setup_collaboration() is used.
        field: str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the study. Default is None.
        fields: list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the study.
            Default is None.

        Returns
        -------
        dict
            Containing the new study information
        """
        if collaboration is None:
            collaboration = self.parent.collaboration_id
            if not collaboration:
                raise ValueError(
                    "No collaboration id provided. Please provide the collaboration as "
                    "argument or use `client.setup_collaboration`."
                )
        return self.parent.request(
            "study",
            method="post",
            json={
                "name": name,
                "organization_ids": organizations,
                "collaboration_id": collaboration,
            },
        )

    def delete(self, id_: int = None) -> None:
        """Deletes a study

        Parameters
        ----------
        id_ : int
            Id of the study you want to delete
        """
        res = self.parent.request(f"study/{id_}", method="delete")
        self.parent.log.info(f"--> {res.get('msg')}")

    @post_filtering(iterable=False)
    def update(
        self,
        id_: int,
        name: str = None,
        organizations: List[int] = None,
    ) -> dict:
        """
        Update study information

        Parameters
        ----------
        id_ : int
            Id of the study you want to update.
        name : str, optional
            New name of the study
        organizations : list[int], optional
            New list of organization ids which participate in the study
        field: str, optional
            Which data field to keep in the returned dict. For instance, "field='name'"
            will only return the name of the study. Default is None.
        fields: list[str], optional
            Which data fields to keep in the returned dict. For instance,
            "fields=['name', 'id']" will only return the name and id of the study.
            Default is None.

        Returns
        -------
        dict
            Containing the updated study information
        """
        data = {"name": name, "organization_ids": organizations}
        data = self._clean_update_data(data)
        return self.parent.request(f"study/{id_}", method="patch", json=data)

    def add_organization(self, organization: int, study: int = None) -> List[dict]:
        """
        Add an organization to a study

        Parameters
        ----------
        organization : int
            Id of the organization you want to add to the study
        study : int, optional
            Id of the study you want to add the organization to.

        Returns
        -------
        list[dict]
            Containing the updated list of organizations in the study
        """
        return self.parent.request(
            f"study/{study}/organization",
            method="post",
            json={"id": organization},
        )

    def remove_organization(self, organization: int, study: int = None) -> List[dict]:
        """
        Remove an organization from a study

        Parameters
        ----------
        organization : int
            Id of the organization you want to remove from the study
        study : int, optional
            Id of the study you want to remove the organization from

        Returns
        -------
        list[dict]
            Containing the updated list of organizations in the study
        """
        return self.parent.request(
            f"study/{study}/organization",
            method="delete",
            json={"id": organization},
        )



import base64
import json

from unittest import TestCase
from unittest.mock import patch, MagicMock

from vantage6.client import UserClient
from vantage6.common.globals import STRING_ENCODING

# Mock server
HOST = "mock_server"
PORT = 1234

# Mock credentials
FAKE_USERNAME = "vantage6_test"
FAKE_PASSWORD = "secretpassword"
FAKE_ID = 1

TASK_NAME = "test-task"
TASK_IMAGE = "mock-image"
COLLABORATION_ID = 1
ORGANIZATION_IDS = [1]
SAMPLE_INPUT = {"method": "test-task"}
FAKE_NAME = "john doe"


class TestClient(TestCase):
    def test_post_task(self):
        post_input = TestClient.post_task_on_mock_client(SAMPLE_INPUT)
        decoded_input = base64.b64decode(post_input)
        assert b'{"method": "test-task"}' == decoded_input

    def test_get_results(self):
        mock_result = json.dumps({"some_key": "some_value"}).encode()

        results = TestClient._receive_results_on_mock_client(mock_result)

        assert results == [{"result": {"some_key": "some_value"}}]

    def test_parse_arg_databases(self):
        dbs_in = [{"label": "dblabel"}]
        dbs_out = UserClient.Task._parse_arg_databases(dbs_in)
        self.assertEqual(dbs_in, dbs_out)

        dbs_in = [{"label": "dblabel"}, {"label": "dblabel2"}, {"label": "dblabel3"}]
        dbs_out = UserClient.Task._parse_arg_databases(dbs_in)
        self.assertEqual(dbs_in, dbs_out)

        dbs_in = "labelstr"
        dbs_out = UserClient.Task._parse_arg_databases(dbs_in)
        self.assertEqual(dbs_out, [{"label": "labelstr"}])

        dbs_in = [{"label": "dblabel"}, {"label": "dblabel2"}, "single_label"]
        with self.assertRaisesRegex(ValueError, "list of dict"):
            dbs_out = UserClient.Task._parse_arg_databases(dbs_in)

        dbs_in = [{"nolabel": "dblabel"}]
        with self.assertRaisesRegex(ValueError, "label"):
            dbs_out = UserClient.Task._parse_arg_databases(dbs_in)

        dbs_in = [{"label": "bad-label"}]
        with self.assertRaisesRegex(ValueError, "(?i)Invalid label"):
            dbs_out = UserClient.Task._parse_arg_databases(dbs_in)

        dbs_in = "1badlabel"
        with self.assertRaisesRegex(ValueError, "(?i)Invalid label"):
            dbs_out = UserClient.Task._parse_arg_databases(dbs_in)

    @staticmethod
    def post_task_on_mock_client(input_) -> dict[str, any]:
        mock_requests = MagicMock()
        mock_requests.get.return_value.status_code = 200
        mock_requests.post.return_value.status_code = 200

        mock_jwt = TestClient._create_mock_jwt()
        with patch.multiple("vantage6.client", requests=mock_requests, jwt=mock_jwt):
            client = TestClient.setup_client()

            client.task.create(
                name=TASK_NAME,
                image=TASK_IMAGE,
                collaboration_id=COLLABORATION_ID,
                organization_ids=ORGANIZATION_IDS,
                input_=input_,
            )

            # In a request.post call, json is provided with the keyword
            # argument 'json'. call_args provides a tuple with positional
            # arguments followed by a dict with positional arguments
            post_content = mock_requests.post.call_args[1]["json"]

            post_input = post_content["organizations"][0]["input"]

        return post_input

    @staticmethod
    def _receive_results_on_mock_client(mock_result):
        mock_result = base64.b64encode(mock_result).decode(STRING_ENCODING)
        mock_result_response = [{"result": mock_result}]
        mock_jwt = TestClient._create_mock_jwt()

        mock_requests = MagicMock()
        mock_requests.get.return_value.status_code = 200
        mock_requests.post.return_value.status_code = 200

        user = {"id": FAKE_ID, "firstname": "naam", "organization": {"id": FAKE_ID}}
        organization = {"id": FAKE_ID, "name": FAKE_NAME}

        # The client will first send a post request for authentication, then
        # for retrieving results.
        mock_requests.get.return_value.json.side_effect = [
            user,
            organization,
            mock_result_response,
        ]

        with patch.multiple("vantage6.client", requests=mock_requests, jwt=mock_jwt):
            client = TestClient.setup_client()

            results = client.result.from_task(task_id=FAKE_ID)

            return results

    @staticmethod
    def setup_client() -> UserClient:
        client = UserClient(HOST, PORT)
        client.authenticate(FAKE_USERNAME, FAKE_PASSWORD)
        client.setup_encryption(None)
        return client

    @staticmethod
    def _create_mock_jwt() -> MagicMock:
        mock_jwt = MagicMock()
        mock_jwt.decode.return_value = {"sub": FAKE_ID}
        return mock_jwt



from typing import Any, List
from vantage6.client.filter import post_filtering
from vantage6.common.client.client_base import ClientBase


class StoreRoleSubClient(ClientBase.SubClient):
    """Subclient for the roles from the algorithm store."""

    @post_filtering(iterable=True)
    def list(
        self,
        name: str = None,
        description: str = None,
        user: int = None,
        page: int = 1,
        per_page: int = 10,
    ) -> List[dict]:
        """
        List algorithms

        Parameters
        ----------
        name : str, optional
            Filter by name (with LIKE operator).
        description : str, optional
            Filter by description (with LIKE operator).
        user : int, optional
            Filter by user id.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the roles. Default is None.
        fields : list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            roles. Default is None.
        filter_ : tuple, optional
            Filter the result on key-value pairs. For instance,
            "filter_=('name', 'my_name')" will only return the roles with the
            name 'my_name'. Default is None.
        filters : list[tuple], optional
            Filter the result on multiple key-value pairs. For instance,
            "filters=[('name', 'my_name'), ('id', 1)]" will only return the
            roles with the name 'my_name' and id 1. Default is None.
        page : int, optional
            Page number for pagination (default=1)
        per_page : int, optional
            Number of items per page (default=10)

        Returns
        -------
        list[dict]
            List of roles
        """
        params = {
            "name": name,
            "description": description,
            "user_id": user,
            "page": page,
            "per_page": per_page,
        }
        return self.parent.request(
            "role",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            params=params,
        )

    @post_filtering(iterable=False)
    def create(self, name: str, description: str, rules: List[int]) -> dict[str, Any]:
        """
        Create a new role

        Parameters
        ----------
        name : str
            Name of the role.
        description : str
            Description of the role.
        rules : list[int]
            List of rule IDs associated with the role.

        Returns
        -------
        dict
            The created role.
        """
        data = {
            "name": name,
            "description": description,
            "rules": rules,
        }
        return self.parent.request(
            "role",
            method="post",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=data,
        )

    @post_filtering(iterable=False)
    def edit(
        self,
        role_id: int,
        name: str = None,
        description: str = None,
        rules: List[int] = None,
    ) -> dict[str, Any]:
        """
        Edit an existing role

        Parameters
        ----------
        role_id : int
            ID of the role to edit.
        name : str, optional
            New name of the role.
        description : str, optional
            New description of the role.
        rules : list[int], optional
            New list of rule IDs associated with the role.
            CAUTION! This will not *add* rules but replace them. If
            you remove rules from your own role you lose access. By
            default None

        Returns
        -------
        dict
            The updated role.
        """
        data = {}
        if name is not None:
            data["name"] = name
        if description is not None:
            data["description"] = description
        if rules is not None:
            data["rules"] = rules

        return self.parent.request(
            f"role/{role_id}",
            method="patch",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
            json=data,
        )

    def delete(self, role_id: int) -> None:
        """
        Delete an existing role

        Parameters
        ----------
        role_id : int
            ID of the role to delete.

        Returns
        -------
        None
        """
        self.parent.request(
            f"role/{role_id}",
            method="delete",
            is_for_algorithm_store=True,
            headers=self.parent.util._get_server_url_header(),
        )
